@@ -1,6 +1,5 @@
 /* Supporting functions for resolving DATA statement.
-   Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-   Free Software Foundation, Inc.
+   Copyright (C) 2002-2017 Free Software Foundation, Inc.
    Contributed by Lifang Zeng <zlf605@hotmail.com>
 
 This file is part of GCC.
@@ -35,6 +34,7 @@ along with GCC; see the file COPYING3.  If not see
 
 #include "config.h"
 #include "system.h"
+#include "coretypes.h"
 #include "gfortran.h"
 #include "data.h"
 #include "constructor.h"
@@ -65,6 +65,7 @@ get_array_index (gfc_array_ref *ar, mpz_t *offset)
 	gfc_error ("non-constant array in DATA statement %L", &ar->where);
 
       mpz_set (tmp, e->value.integer);
+      gfc_free_expr (e);
       mpz_sub (tmp, tmp, ar->as->lower[i]->value.integer);
       mpz_mul (tmp, tmp, delta);
       mpz_add (*offset, tmp, *offset);
@@ -103,8 +104,9 @@ static gfc_expr *
 create_character_initializer (gfc_expr *init, gfc_typespec *ts,
 			      gfc_ref *ref, gfc_expr *rvalue)
 {
-  int len, start, end;
+  int len, start, end, tlen;
   gfc_char_t *dest;
+  bool alloced_init = false;
 	    
   gfc_extract_int (ts->u.cl->length, &len);
 
@@ -113,6 +115,7 @@ create_character_initializer (gfc_expr *init, gfc_typespec *ts,
       /* Create a new initializer.  */
       init = gfc_get_character_expr (ts->kind, NULL, NULL, len);
       init->ts = *ts;
+      alloced_init = true;
     }
 
   dest = init->value.character.string;
@@ -128,17 +131,23 @@ create_character_initializer (gfc_expr *init, gfc_typespec *ts,
       start_expr = gfc_copy_expr (ref->u.ss.start);
       end_expr = gfc_copy_expr (ref->u.ss.end);
 
-      if ((gfc_simplify_expr (start_expr, 1) == FAILURE)
-	  || (gfc_simplify_expr (end_expr, 1)) == FAILURE)
+      if ((!gfc_simplify_expr(start_expr, 1))
+	  || !(gfc_simplify_expr(end_expr, 1)))
 	{
 	  gfc_error ("failure to simplify substring reference in DATA "
 		     "statement at %L", &ref->u.ss.start->where);
+	  gfc_free_expr (start_expr);
+	  gfc_free_expr (end_expr);
+	  if (alloced_init)
+	    gfc_free_expr (init);
 	  return NULL;
 	}
 
       gfc_extract_int (start_expr, &start);
+      gfc_free_expr (start_expr);
       start--;
       gfc_extract_int (end_expr, &end);
+      gfc_free_expr (end_expr);
     }
   else
     {
@@ -153,12 +162,22 @@ create_character_initializer (gfc_expr *init, gfc_typespec *ts,
   else
     len = rvalue->value.character.length;
 
-  if (len > end - start)
+  tlen = end - start;
+  if (len > tlen)
     {
-      gfc_warning_now ("Initialization string starting at %L was "
-		       "truncated to fit the variable (%d/%d)",
-		       &rvalue->where, end - start, len);
-      len = end - start;
+      if (tlen < 0)
+	{
+	  gfc_warning_now (0, "Unused initialization string at %L because "
+			   "variable has zero length", &rvalue->where);
+	  len = 0;
+	}
+      else
+	{
+	  gfc_warning_now (0, "Initialization string at %L was truncated to "
+			   "fit the variable (%d/%d)", &rvalue->where,
+			   tlen, len);
+	  len = tlen;
+	}
     }
 
   if (rvalue->ts.type == BT_HOLLERITH)
@@ -172,7 +191,7 @@ create_character_initializer (gfc_expr *init, gfc_typespec *ts,
 	    len * sizeof (gfc_char_t));
 
   /* Pad with spaces.  Substrings will already be blanked.  */
-  if (len < end - start && ref == NULL)
+  if (len < tlen && ref == NULL)
     gfc_wide_memset (&dest[start + len], ' ', end - (start + len));
 
   if (rvalue->ts.type == BT_HOLLERITH)
@@ -193,13 +212,13 @@ create_character_initializer (gfc_expr *init, gfc_typespec *ts,
    consecutive values in LVALUE the same value in RVALUE.  In that case,
    LVALUE must refer to a full array, not an array section.  */
 
-gfc_try
+bool
 gfc_assign_data_value (gfc_expr *lvalue, gfc_expr *rvalue, mpz_t index,
 		       mpz_t *repeat)
 {
   gfc_ref *ref;
   gfc_expr *init;
-  gfc_expr *expr;
+  gfc_expr *expr = NULL;
   gfc_constructor *con;
   gfc_constructor *last_con;
   gfc_symbol *symbol;
@@ -244,7 +263,7 @@ gfc_assign_data_value (gfc_expr *lvalue, gfc_expr *rvalue, mpz_t index,
 
 	  if (init && expr->expr_type != EXPR_ARRAY)
 	    {
-	      gfc_error ("'%s' at %L already is initialized at %L",
+	      gfc_error ("%qs at %L already is initialized at %L",
 			 lvalue->symtree->n.sym->name, &lvalue->where,
 			 &init->where);
 	      goto abort;
@@ -280,7 +299,7 @@ gfc_assign_data_value (gfc_expr *lvalue, gfc_expr *rvalue, mpz_t index,
 			  && ref->next == NULL);
 	      mpz_init_set (end, offset);
 	      mpz_add (end, end, *repeat);
-	      if (spec_size (ref->u.ar.as, &size) == SUCCESS)
+	      if (spec_size (ref->u.ar.as, &size))
 		{
 		  if (mpz_cmp (end, size) > 0)
 		    {
@@ -314,10 +333,10 @@ gfc_assign_data_value (gfc_expr *lvalue, gfc_expr *rvalue, mpz_t index,
 		  exprd = (LOCATION_LINE (con->expr->where.lb->location)
 			   > LOCATION_LINE (rvalue->where.lb->location))
 			  ? con->expr : rvalue;
-		  if (gfc_notify_std (GFC_STD_GNU,"Extension: "
-				      "re-initialization of '%s' at %L",
-				      symbol->name, &exprd->where) == FAILURE)
-		    return FAILURE;
+		  if (gfc_notify_std (GFC_STD_GNU,
+				      "re-initialization of %qs at %L",
+				      symbol->name, &exprd->where) == false)
+		    return false;
 		}
 
 	      while (con != NULL)
@@ -369,7 +388,7 @@ gfc_assign_data_value (gfc_expr *lvalue, gfc_expr *rvalue, mpz_t index,
 	  else
 	    {
 	      mpz_t size;
-	      if (spec_size (ref->u.ar.as, &size) == SUCCESS)
+	      if (spec_size (ref->u.ar.as, &size))
 		{
 		  if (mpz_cmp (offset, size) >= 0)
 		    {
@@ -464,8 +483,11 @@ gfc_assign_data_value (gfc_expr *lvalue, gfc_expr *rvalue, mpz_t index,
 
   if (ref || last_ts->type == BT_CHARACTER)
     {
-      if (lvalue->ts.u.cl->length == NULL && !(ref && ref->u.ss.length != NULL))
-	return FAILURE;
+      /* An initializer has to be constant.  */
+      if (rvalue->expr_type != EXPR_CONSTANT
+	  || (lvalue->ts.u.cl->length == NULL
+	      && !(ref && ref->u.ss.length != NULL)))
+	return false;
       expr = create_character_initializer (init, last_ts, ref, rvalue);
     }
   else
@@ -480,10 +502,10 @@ gfc_assign_data_value (gfc_expr *lvalue, gfc_expr *rvalue, mpz_t index,
 	  expr = (LOCATION_LINE (init->where.lb->location)
 		  > LOCATION_LINE (rvalue->where.lb->location))
 	       ? init : rvalue;
-	  if (gfc_notify_std (GFC_STD_GNU,"Extension: "
-			      "re-initialization of '%s' at %L",
-			      symbol->name, &expr->where) == FAILURE)
-	    return FAILURE;
+	  if (gfc_notify_std (GFC_STD_GNU,
+			      "re-initialization of %qs at %L",
+			      symbol->name, &expr->where) == false)
+	    return false;
 	}
 
       expr = gfc_copy_expr (rvalue);
@@ -496,11 +518,13 @@ gfc_assign_data_value (gfc_expr *lvalue, gfc_expr *rvalue, mpz_t index,
   else
     last_con->expr = expr;
 
-  return SUCCESS;
+  return true;
 
 abort:
+  if (!init)
+    gfc_free_expr (expr);
   mpz_clear (offset);
-  return FAILURE;
+  return false;
 }
 
 
