@@ -1,5 +1,5 @@
 /* Generate code from to output assembler insns as recognized from rtl.
-   Copyright (C) 1987-2015 Free Software Foundation, Inc.
+   Copyright (C) 1987-2017 Free Software Foundation, Inc.
 
 This file is part of GCC.
 
@@ -103,17 +103,6 @@ static char general_mem[] = { TARGET_MEM_CONSTRAINT, 0 };
 static int n_occurrences		(int, const char *);
 static const char *strip_whitespace	(const char *);
 
-/* insns in the machine description are assigned sequential code numbers
-   that are used by insn-recog.c (produced by genrecog) to communicate
-   to insn-output.c (produced by this program).  */
-
-static int next_code_number;
-
-/* This counts all definitions in the md file,
-   for the sake of error messages.  */
-
-static int next_index_number;
-
 /* This counts all operands used in the md file.  The first is null.  */
 
 static int next_operand_number = 1;
@@ -159,10 +148,8 @@ struct data
   struct data *next;
   const char *name;
   const char *template_code;
+  file_location loc;
   int code_number;
-  int index_number;
-  const char *filename;
-  int lineno;
   int n_generator_args;		/* Number of arguments passed to generator */
   int n_operands;		/* Number of operands this insn recognizes */
   int n_dups;			/* Number times match_dup appears in pattern */
@@ -172,15 +159,12 @@ struct data
   struct operand_data operand[MAX_MAX_OPERANDS];
 };
 
-/* A dummy insn, for CODE_FOR_nothing.  */
-static struct data nothing;
-
 /* This variable points to the first link in the insn chain.  */
-static struct data *idata = &nothing;
+static struct data *idata;
 
 /* This variable points to the end of the insn chain.  This is where
    everything relevant from the machien description is appended to.  */
-static struct data **idata_end = &nothing.next;
+static struct data **idata_end;
 
 
 static void output_prologue (void);
@@ -194,17 +178,13 @@ static void place_operands (struct data *);
 static void process_template (struct data *, const char *);
 static void validate_insn_alternatives (struct data *);
 static void validate_insn_operands (struct data *);
-static void gen_insn (rtx, int);
-static void gen_peephole (rtx, int);
-static void gen_expand (rtx, int);
-static void gen_split (rtx, int);
 
 struct constraint_data
 {
   struct constraint_data *next_this_letter;
-  int lineno;
+  file_location loc;
   unsigned int namelen;
-  const char name[1];
+  char name[1];
 };
 
 /* All machine-independent constraint characters (except digits) that
@@ -214,8 +194,8 @@ static const char indep_constraints[] = ",=+%*?!^$#&g";
 static struct constraint_data *
 constraints_by_letter_table[1 << CHAR_BIT];
 
-static int mdep_constraint_len (const char *, int, int);
-static void note_constraint (rtx, int);
+static int mdep_constraint_len (const char *, file_location, int);
+static void note_constraint (md_rtx_info *);
 
 static void
 output_prologue (void)
@@ -226,33 +206,20 @@ output_prologue (void)
   printf ("#include \"config.h\"\n");
   printf ("#include \"system.h\"\n");
   printf ("#include \"coretypes.h\"\n");
-  printf ("#include \"tm.h\"\n");
-  printf ("#include \"flags.h\"\n");
-  printf ("#include \"ggc.h\"\n");
-  printf ("#include \"hash-set.h\"\n");
-  printf ("#include \"machmode.h\"\n");
-  printf ("#include \"vec.h\"\n");
-  printf ("#include \"double-int.h\"\n");
-  printf ("#include \"input.h\"\n");
-  printf ("#include \"alias.h\"\n");
-  printf ("#include \"symtab.h\"\n");
-  printf ("#include \"wide-int.h\"\n");
-  printf ("#include \"inchash.h\"\n");
+  printf ("#include \"backend.h\"\n");
+  printf ("#include \"predict.h\"\n");
   printf ("#include \"tree.h\"\n");
+  printf ("#include \"rtl.h\"\n");
+  printf ("#include \"flags.h\"\n");
+  printf ("#include \"alias.h\"\n");
   printf ("#include \"varasm.h\"\n");
   printf ("#include \"stor-layout.h\"\n");
   printf ("#include \"calls.h\"\n");
-  printf ("#include \"rtl.h\"\n");
-  printf ("#include \"hashtab.h\"\n");
-  printf ("#include \"hard-reg-set.h\"\n");
-  printf ("#include \"function.h\"\n");
-  printf ("#include \"statistics.h\"\n");
-  printf ("#include \"real.h\"\n");
-  printf ("#include \"fixed-value.h\"\n");
   printf ("#include \"insn-config.h\"\n");
   printf ("#include \"expmed.h\"\n");
   printf ("#include \"dojump.h\"\n");
   printf ("#include \"explow.h\"\n");
+  printf ("#include \"memmodel.h\"\n");
   printf ("#include \"emit-rtl.h\"\n");
   printf ("#include \"stmt.h\"\n");
   printf ("#include \"expr.h\"\n");
@@ -266,7 +233,6 @@ output_prologue (void)
   printf ("#include \"output.h\"\n");
   printf ("#include \"target.h\"\n");
   printf ("#include \"tm-constrs.h\"\n");
-  printf ("#include \"predict.h\"\n");
 }
 
 static void
@@ -327,7 +293,7 @@ output_insn_data (void)
 
   for (d = idata; d; d = d->next)
     {
-      printf ("  /* %s:%d */\n", d->filename, d->lineno);
+      printf ("  /* %s:%d */\n", d->loc.filename, d->loc.lineno);
       printf ("  {\n");
 
       if (d->name)
@@ -470,11 +436,11 @@ scan_operands (struct data *d, rtx part, int this_address_p,
       opno = XINT (part, 0);
       if (opno >= MAX_MAX_OPERANDS)
 	{
-	  error_with_line (d->lineno, "maximum number of operands exceeded");
+	  error_at (d->loc, "maximum number of operands exceeded");
 	  return;
 	}
       if (d->operand[opno].seen)
-	error_with_line (d->lineno, "repeated operand number %d\n", opno);
+	error_at (d->loc, "repeated operand number %d\n", opno);
 
       d->operand[opno].seen = 1;
       d->operand[opno].mode = GET_MODE (part);
@@ -491,11 +457,11 @@ scan_operands (struct data *d, rtx part, int this_address_p,
       opno = XINT (part, 0);
       if (opno >= MAX_MAX_OPERANDS)
 	{
-	  error_with_line (d->lineno, "maximum number of operands exceeded");
+	  error_at (d->loc, "maximum number of operands exceeded");
 	  return;
 	}
       if (d->operand[opno].seen)
-	error_with_line (d->lineno, "repeated operand number %d\n", opno);
+	error_at (d->loc, "repeated operand number %d\n", opno);
 
       d->operand[opno].seen = 1;
       d->operand[opno].mode = GET_MODE (part);
@@ -513,11 +479,11 @@ scan_operands (struct data *d, rtx part, int this_address_p,
       opno = XINT (part, 0);
       if (opno >= MAX_MAX_OPERANDS)
 	{
-	  error_with_line (d->lineno, "maximum number of operands exceeded");
+	  error_at (d->loc, "maximum number of operands exceeded");
 	  return;
 	}
       if (d->operand[opno].seen)
-	error_with_line (d->lineno, "repeated operand number %d\n", opno);
+	error_at (d->loc, "repeated operand number %d\n", opno);
 
       d->operand[opno].seen = 1;
       d->operand[opno].mode = GET_MODE (part);
@@ -665,7 +631,7 @@ process_template (struct data *d, const char *template_code)
       printf ("output_%d (rtx *operands ATTRIBUTE_UNUSED, rtx_insn *insn ATTRIBUTE_UNUSED)\n",
 	      d->code_number);
       puts ("{");
-      print_md_ptr_loc (template_code);
+      rtx_reader_ptr->print_md_ptr_loc (template_code);
       puts (template_code + 1);
       puts ("}");
     }
@@ -729,8 +695,7 @@ process_template (struct data *d, const char *template_code)
 	      sp = ep + 1;
 
 	  if (sp != ep)
-	    message_with_line (d->lineno,
-			       "trailing whitespace in output template");
+	    message_at (d->loc, "trailing whitespace in output template");
 
 	  while (cp < sp)
 	    {
@@ -753,11 +718,11 @@ process_template (struct data *d, const char *template_code)
 	  i++;
 	}
       if (i == 1)
-	message_with_line (d->lineno,
-			   "'@' is redundant for output template with single alternative");
+	message_at (d->loc, "'@' is redundant for output template with"
+		    " single alternative");
       if (i != d->n_alternatives)
-	error_with_line (d->lineno,
-			 "wrong number of alternatives in the output template");
+	error_at (d->loc, "wrong number of alternatives in the output"
+		  " template");
 
       if (found_star)
 	puts ("      default: gcc_unreachable ();\n    }\n}");
@@ -794,9 +759,8 @@ validate_insn_alternatives (struct data *d)
 	  {
 	    if ((c == '%' || c == '=' || c == '+')
 		&& p != d->operand[start].constraint)
-	      error_with_line (d->lineno,
-			       "character '%c' can only be used at the"
-			       " beginning of a constraint string", c);
+	      error_at (d->loc, "character '%c' can only be used at the"
+			" beginning of a constraint string", c);
 
 	    if (c == '=' || c == '+')
 	      seen_write = true;
@@ -804,10 +768,8 @@ validate_insn_alternatives (struct data *d)
 	    /* Earlyclobber operands must always be marked write-only
 	       or read/write.  */
 	    if (!seen_write && c == '&')
-	      error_with_line (d->lineno,
-			       "earlyclobber operands may not be"
-			       " read-only in alternative %d",
-			       which_alternative);
+	      error_at (d->loc, "earlyclobber operands may not be"
+			" read-only in alternative %d", which_alternative);
 
 	    if (ISSPACE (c) || strchr (indep_constraints, c))
 	      len = 1;
@@ -820,7 +782,7 @@ validate_insn_alternatives (struct data *d)
 		len = q - p;
 	      }
 	    else
-	      len = mdep_constraint_len (p, d->lineno, start);
+	      len = mdep_constraint_len (p, d->loc, start);
 
 	    if (c == ',')
 	      {
@@ -831,17 +793,15 @@ validate_insn_alternatives (struct data *d)
 	    for (i = 1; i < len; i++)
 	      if (p[i] == '\0')
 		{
-		  error_with_line (d->lineno,
-				   "NUL in alternative %d of operand %d",
-				   which_alternative, start);
+		  error_at (d->loc, "NUL in alternative %d of operand %d",
+			    which_alternative, start);
 		  alternative_count_unsure = 1;
 		  break;
 		}
 	      else if (strchr (",#*", p[i]))
 		{
-		  error_with_line (d->lineno,
-				   "'%c' in alternative %d of operand %d",
-				   p[i], which_alternative, start);
+		  error_at (d->loc, "'%c' in alternative %d of operand %d",
+			    p[i], which_alternative, start);
 		  alternative_count_unsure = 1;
 		}
 	  }
@@ -850,9 +810,8 @@ validate_insn_alternatives (struct data *d)
 	    if (n == 0)
 	      n = d->operand[start].n_alternatives;
 	    else if (n != d->operand[start].n_alternatives)
-	      error_with_line (d->lineno,
-			       "wrong number of alternatives in operand %d",
-			       start);
+	      error_at (d->loc, "wrong number of alternatives in operand %d",
+			start);
 	  }
       }
 
@@ -869,7 +828,7 @@ validate_insn_operands (struct data *d)
 
   for (i = 0; i < d->n_operands; ++i)
     if (d->operand[i].seen == 0)
-      error_with_line (d->lineno, "missing operand %d", i);
+      error_at (d->loc, "missing operand %d", i);
 }
 
 static void
@@ -883,7 +842,7 @@ validate_optab_operands (struct data *d)
       && d->name[strlen (d->name) - 1] == '4'
       && d->operand[0].mode == VOIDmode)
     {
-      message_with_line (d->lineno, "missing mode for operand 0 of cstore");
+      message_at (d->loc, "missing mode for operand 0 of cstore");
       have_error = 1;
     }
 }
@@ -893,16 +852,15 @@ validate_optab_operands (struct data *d)
    a hairy output action, output a function for now.  */
 
 static void
-gen_insn (rtx insn, int lineno)
+gen_insn (md_rtx_info *info)
 {
   struct pattern_stats stats;
-  struct data *d = XNEW (struct data);
+  rtx insn = info->def;
+  data *d = new data;
   int i;
 
-  d->code_number = next_code_number;
-  d->index_number = next_index_number;
-  d->filename = read_md_filename;
-  d->lineno = lineno;
+  d->code_number = info->index;
+  d->loc = info->loc;
   if (XSTR (insn, 0)[0])
     d->name = XSTR (insn, 0);
   else
@@ -936,16 +894,14 @@ gen_insn (rtx insn, int lineno)
    If the insn has a hairy output action, output it now.  */
 
 static void
-gen_peephole (rtx peep, int lineno)
+gen_peephole (md_rtx_info *info)
 {
   struct pattern_stats stats;
-  struct data *d = XNEW (struct data);
+  data *d = new data;
   int i;
 
-  d->code_number = next_code_number;
-  d->index_number = next_index_number;
-  d->filename = read_md_filename;
-  d->lineno = lineno;
+  d->code_number = info->index;
+  d->loc = info->loc;
   d->name = 0;
 
   /* Build up the list in the same order as the insns are seen
@@ -959,6 +915,7 @@ gen_peephole (rtx peep, int lineno)
   /* Get the number of operands by scanning all the patterns of the
      peephole optimizer.  But ignore all the rest of the information
      thus obtained.  */
+  rtx peep = info->def;
   for (i = 0; i < XVECLEN (peep, 0); i++)
     scan_operands (d, XVECEXP (peep, 0, i), 0, 0);
 
@@ -976,16 +933,15 @@ gen_peephole (rtx peep, int lineno)
    only for the purposes of `insn_gen_function'.  */
 
 static void
-gen_expand (rtx insn, int lineno)
+gen_expand (md_rtx_info *info)
 {
   struct pattern_stats stats;
-  struct data *d = XNEW (struct data);
+  rtx insn = info->def;
+  data *d = new data;
   int i;
 
-  d->code_number = next_code_number;
-  d->index_number = next_index_number;
-  d->filename = read_md_filename;
-  d->lineno = lineno;
+  d->code_number = info->index;
+  d->loc = info->loc;
   if (XSTR (insn, 0)[0])
     d->name = XSTR (insn, 0);
   else
@@ -1018,62 +974,21 @@ gen_expand (rtx insn, int lineno)
   place_operands (d);
 }
 
-/* Process a define_split just read.  Assign its code number,
-   only for reasons of consistency and to simplify genrecog.  */
-
-static void
-gen_split (rtx split, int lineno)
-{
-  struct pattern_stats stats;
-  struct data *d = XNEW (struct data);
-  int i;
-
-  d->code_number = next_code_number;
-  d->index_number = next_index_number;
-  d->filename = read_md_filename;
-  d->lineno = lineno;
-  d->name = 0;
-
-  /* Build up the list in the same order as the insns are seen
-     in the machine description.  */
-  d->next = 0;
-  *idata_end = d;
-  idata_end = &d->next;
-
-  memset (d->operand, 0, sizeof (d->operand));
-
-  /* Get the number of operands by scanning all the patterns of the
-     split patterns.  But ignore all the rest of the information thus
-     obtained.  */
-  for (i = 0; i < XVECLEN (split, 0); i++)
-    scan_operands (d, XVECEXP (split, 0, i), 0, 0);
-
-  get_pattern_stats (&stats, XVEC (split, 0));
-  d->n_generator_args = 0;
-  d->n_operands = stats.num_insn_operands;
-  d->n_dups = 0;
-  d->n_alternatives = 0;
-  d->template_code = 0;
-  d->output_format = INSN_OUTPUT_FORMAT_NONE;
-
-  place_operands (d);
-}
-
 static void
 init_insn_for_nothing (void)
 {
-  memset (&nothing, 0, sizeof (nothing));
-  nothing.name = "*placeholder_for_nothing";
-  nothing.filename = "<internal>";
+  idata = XCNEW (struct data);
+  new (idata) data ();
+  idata->name = "*placeholder_for_nothing";
+  idata->loc = file_location ("<internal>", 0, 0);
+  idata_end = &idata->next;
 }
 
-extern int main (int, char **);
+extern int main (int, const char **);
 
 int
-main (int argc, char **argv)
+main (int argc, const char **argv)
 {
-  rtx desc;
-
   progname = "genoutput";
 
   init_insn_for_nothing ();
@@ -1082,49 +997,36 @@ main (int argc, char **argv)
     return (FATAL_EXIT_CODE);
 
   output_prologue ();
-  next_index_number = 0;
 
   /* Read the machine description.  */
 
-  while (1)
-    {
-      int line_no;
-
-      desc = read_md_rtx (&line_no, &next_code_number);
-      if (desc == NULL)
+  md_rtx_info info;
+  while (read_md_rtx (&info))
+    switch (GET_CODE (info.def))
+      {
+      case DEFINE_INSN:
+	gen_insn (&info);
 	break;
 
-      switch (GET_CODE (desc))
-	{
-	case DEFINE_INSN:
-	  gen_insn (desc, line_no);
-	  break;
+      case DEFINE_PEEPHOLE:
+	gen_peephole (&info);
+	break;
 
-	case DEFINE_PEEPHOLE:
-	  gen_peephole (desc, line_no);
-	  break;
+      case DEFINE_EXPAND:
+	gen_expand (&info);
+	break;
 
-	case DEFINE_EXPAND:
-	  gen_expand (desc, line_no);
-	  break;
+      case DEFINE_CONSTRAINT:
+      case DEFINE_REGISTER_CONSTRAINT:
+      case DEFINE_ADDRESS_CONSTRAINT:
+      case DEFINE_MEMORY_CONSTRAINT:
+      case DEFINE_SPECIAL_MEMORY_CONSTRAINT:
+	note_constraint (&info);
+	break;
 
-	case DEFINE_SPLIT:
-	case DEFINE_PEEPHOLE2:
-	  gen_split (desc, line_no);
-	  break;
-
-	case DEFINE_CONSTRAINT:
-	case DEFINE_REGISTER_CONSTRAINT:
-	case DEFINE_ADDRESS_CONSTRAINT:
-	case DEFINE_MEMORY_CONSTRAINT:
-	  note_constraint (desc, line_no);
-	  break;
-
-	default:
-	  break;
-	}
-      next_index_number++;
-    }
+      default:
+	break;
+      }
 
   printf ("\n\n");
   output_operand_data ();
@@ -1174,15 +1076,14 @@ strip_whitespace (const char *s)
   return q;
 }
 
-/* Record just enough information about a constraint to allow checking
-   of operand constraint strings above, in validate_insn_alternatives.
-   Does not validate most properties of the constraint itself; does
-   enforce no duplicate names, no overlap with MI constraints, and no
-   prefixes.  EXP is the define_*constraint form, LINENO the line number
-   reported by the reader.  */
+/* Record just enough information about the constraint in *INFO to allow
+   checking of operand constraint strings above, in validate_insn_alternatives.
+   Does not validate most properties of the constraint itself; does enforce
+   no duplicate names, no overlap with MI constraints, and no prefixes.  */
 static void
-note_constraint (rtx exp, int lineno)
+note_constraint (md_rtx_info *info)
 {
+  rtx exp = info->def;
   const char *name = XSTR (exp, 0);
   struct constraint_data **iter, **slot, *new_cdata;
 
@@ -1193,12 +1094,12 @@ note_constraint (rtx exp, int lineno)
   if (strchr (indep_constraints, name[0]))
     {
       if (name[1] == '\0')
-	error_with_line (lineno, "constraint letter '%s' cannot be "
-			 "redefined by the machine description", name);
+	error_at (info->loc, "constraint letter '%s' cannot be "
+		  "redefined by the machine description", name);
       else
-	error_with_line (lineno, "constraint name '%s' cannot be defined by "
-			 "the machine description, as it begins with '%c'",
-			 name, name[0]);
+	error_at (info->loc, "constraint name '%s' cannot be defined by "
+		  "the machine description, as it begins with '%c'",
+		  name, name[0]);
       return;
     }
 
@@ -1215,29 +1116,31 @@ note_constraint (rtx exp, int lineno)
 
       if (!strcmp ((*iter)->name, name))
 	{
-	  error_with_line (lineno, "redefinition of constraint '%s'", name);
-	  message_with_line ((*iter)->lineno, "previous definition is here");
+	  error_at (info->loc, "redefinition of constraint '%s'", name);
+	  message_at ((*iter)->loc, "previous definition is here");
 	  return;
 	}
       else if (!strncmp ((*iter)->name, name, (*iter)->namelen))
 	{
-	  error_with_line (lineno, "defining constraint '%s' here", name);
-	  message_with_line ((*iter)->lineno, "renders constraint '%s' "
-			     "(defined here) a prefix", (*iter)->name);
+	  error_at (info->loc, "defining constraint '%s' here", name);
+	  message_at ((*iter)->loc, "renders constraint '%s' "
+		      "(defined here) a prefix", (*iter)->name);
 	  return;
 	}
       else if (!strncmp ((*iter)->name, name, namelen))
 	{
-	  error_with_line (lineno, "constraint '%s' is a prefix", name);
-	  message_with_line ((*iter)->lineno, "of constraint '%s' "
-			     "(defined here)", (*iter)->name);
+	  error_at (info->loc, "constraint '%s' is a prefix", name);
+	  message_at ((*iter)->loc, "of constraint '%s' "
+		      "(defined here)", (*iter)->name);
 	  return;
 	}
     }
-  new_cdata = XNEWVAR (struct constraint_data, sizeof (struct constraint_data) + namelen);
+  new_cdata = XNEWVAR (struct constraint_data,
+		       sizeof (struct constraint_data) + namelen);
+  new (new_cdata) constraint_data ();
   strcpy (CONST_CAST (char *, new_cdata->name), name);
   new_cdata->namelen = namelen;
-  new_cdata->lineno = lineno;
+  new_cdata->loc = info->loc;
   new_cdata->next_this_letter = *slot;
   *slot = new_cdata;
 }
@@ -1247,7 +1150,7 @@ note_constraint (rtx exp, int lineno)
    is no such constraint.  Does not expect to be called for generic
    constraints.  */
 static int
-mdep_constraint_len (const char *s, int lineno, int opno)
+mdep_constraint_len (const char *s, file_location loc, int opno)
 {
   struct constraint_data *p;
 
@@ -1258,9 +1161,8 @@ mdep_constraint_len (const char *s, int lineno, int opno)
       if (!strncmp (s, p->name, p->namelen))
 	return p->namelen;
 
-  error_with_line (lineno,
-		   "error: undefined machine-specific constraint "
-		   "at this point: \"%s\"", s);
-  message_with_line (lineno, "note:  in operand %d", opno);
+  error_at (loc, "error: undefined machine-specific constraint "
+	    "at this point: \"%s\"", s);
+  message_at (loc, "note:  in operand %d", opno);
   return 1; /* safe */
 }
