@@ -1,5 +1,6 @@
-/* { dg-do run } */
+/* { dg-do run { target openacc_nvidia_accel_selected } } */
 
+#include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <openacc.h>
@@ -21,6 +22,7 @@ main (int argc, char **argv)
 
   d = acc_malloc (N);
 
+  fprintf (stderr, "CheCKpOInT\n");
   acc_memcpy_to_device (0, h, N);
 
   memset (&h[0], 0, N);
@@ -40,4 +42,6 @@ main (int argc, char **argv)
   return 0;
 }
 
-/* { dg-shouldfail "libgomp: invalid device address" } */
+/* { dg-output "CheCKpOInT(\n|\r\n|\r).*" } */
+/* { dg-output "invalid device address" } */
+/* { dg-shouldfail "" } */
