@@ -1,6 +1,6 @@
-// 2003-04-30  Petur Runolfsson <peturr02@ru.is>
+// 2000-08-02 bkoz
 
-// Copyright (C) 2003-2015 Free Software Foundation, Inc.
+// Copyright (C) 2000-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -17,30 +17,20 @@
 // with this library; see the file COPYING3.  If not see
 // <http://www.gnu.org/licenses/>.
 
-#include <testsuite_hooks.h>
-#include <cstdio>
 #include <iostream>
 
-void test02()
+// libstdc++/6648
+// Interactive tests: each one (run alone) must terminate upon a single '\n'.
+void test08()
 {
-  using namespace std;
-
-  bool test __attribute__((unused)) = true;
-
-  int c1 = fgetc(stdin);
-  int c2 = cin.rdbuf()->sputbackc(c1);
-  VERIFY( c2 == c1 );
-  
-  int c3 = fgetc(stdin);
-  VERIFY( c3 == c1 );
-  ungetc(c3, stdin);
-  
-  int c4 = cin.rdbuf()->sgetc();
-  VERIFY( c4 == c3 );
+  char buff[2048];
+  std::cout << "Enter name: ";
+  std::cin.getline(buff, 2048);
 }
 
-int main()
+int 
+main()
 {
-  test02();
+  test08();
   return 0;
 }
