@@ -1,9 +1,9 @@
-// Copyright (C) 2005 Free Software Foundation
+// Copyright (C) 2005-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -12,9 +12,8 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
-// USA.
+// with this library; see the file COPYING3.  If not see
+// <http://www.gnu.org/licenses/>.
 
 // TR1 2.2.2 Template class shared_ptr [tr.util.smartptr.shared]
 
@@ -29,8 +28,6 @@ struct A { };
 int
 test01()
 {
-  bool test __attribute__((unused)) = true;
-
   A * const a1 = new A;
   A * const a2 = new A;
   std::tr1::shared_ptr<A> p1(a1);
@@ -42,7 +39,7 @@ test01()
   return 0;
 }
 
-int 
+int
 main()
 {
   test01();
