@@ -1,6 +1,6 @@
 // 2005-08-26  Paolo Carlini  <pcarlini@suse.de>
 //
-// Copyright (C) 2005, 2009 Free Software Foundation, Inc.
+// Copyright (C) 2005-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -23,9 +23,8 @@
 #include <testsuite_hooks.h>
 
 void
-test01() 
-{ 
-  bool test __attribute__((unused)) = true;
+test01()
+{
   using namespace std::tr1;
 
   {
